#pragma rtGlobals=3		// Use modern global access method and strict wave access.

//This version of the program is built from OrthogonalFingerprintingSimulations_MC_8_comparisonInternalStandard
//by Jorge Alegre-Cebollada (CNIC, Spain, jalegre@cnic.es, Twitter: @AlegreCebollada)
//Written for Igor 6
//To run the program, compile. Under Macros, click Monte Carlo panel and Initialize
//Input experiments and number of events from Monte Carlo (MC) panel or waves in Table 1
//Reference protein is consider to have the same number of experiment and events than protein of interest
//Output in command window includes simulations settings, mechanical parameters for proteins A, B and reference (if internal standard is used)
//with associated SD and RSD. Two plots are also generated: 
//1. Distributions of mean unfolding forces and difference in mean unfolding force
//2. Actual values of mean unfolding forces and differenes in mean unfolding forces for each cycle of simulation. This plot is useful to detect faulty simulation cycles
//      that produce highly divergent results due to wrong fitting. Results from faulty cycles need to be removed manually and RSD recalculated using wavestats function

// More info can be found in:

 //Carolina Pimenta-Lopes, Carmen Suay-Corredera, Diana Velzquez-Carreras, David Snchez-Ortiz, Jorge Alegre-Cebollada (2019). 
 //Concurrent Atomic Force Spectroscopy. Communications Physics 2, 91. (2019).

Menu "Macros"
	"Monte Carlo Panel", MonteCarloSimulations()
	"Initialize",  InitializeMC ()
End

Function InitializeMC ()
Variable/G root:G_Thermal=4.11 				 //pNnm
Variable/G root:G_Force=0     					// pN
Variable/G root:G_alpha0=0.01   			 	// s-1 1st protein - Prot A (e.g.: WT)
Variable/G root:G_Dx=0.2         					 //nm
Variable/G root:G_alpha0_b=0.01   			 	// s-1 2nd protein	- Prot B (e.g.: Mut)
Variable/G root:G_Dx_b=0.2         				 //nm
Variable/G root:G_alpha0_c=0.01   			 	// s-1 Internal Standard
Variable/G root:G_Dx_c=0.2         				 //nm
Variable/G root:G_Time_Step_Interval=0.0001	 //s
Variable/G root:G_Ramp=40 					//pN/s
Variable/G root:G_Number_Of_Cycles=1000
Variable/G root:G_Number_Of_Events=100		//events for 1st protein
Variable/G root:G_Number_Of_Events_b=100	//events for 2nd protein
Variable/G root:G_Number_Of_Experiments=1
Variable/G root:G_Binsize = 25

Variable/G root:G_uncertainty=3.6				//uncertainty in Force, in RSD and %
Variable/G root:G_mode=1						//traditional (1, variation in SC), concurrent (0, no variation in SC within the same experiment)
Variable/G root:G_averagingmethod=1			//0: schlierf, 1: gaussian, 2: mean, 4:error please select averaging method
Variable/G root:G_inputtable=0					//0: no input table; 1: input table for number of experiments and number of events
Variable/G root:G_internalstandard_mode=0		//1: traditional with internal standard strategy

CheckBox traditional,value= 1					//reinitialize mode of averaging and traditional/concurrent
CheckBox concurrent, value =0
CheckBox inputtable, value =0
CheckBox internalstandard, value=0

CheckBox Schlierf,value= 0
CheckBox Gaussian,value= 1
CheckBox Average,value= 0	

Make/O/N=1 neventsProtA, neventsProtB
neventsProtA = 100
neventsProtB = 100
edit neventsProtA, neventsProtB

end

Window MonteCarloSimulations() : Panel	
	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(500,600,1000,850)
	SetDrawLayer UserBack
	GroupBox box1,pos={6,5},size={488,233},title="RSD calculator - Protein Nanomechanics"
	GroupBox box1,font="Times New Roman",fSize=12
	Button button1,pos={20,160},size={75,28},proc=InitMC,title="Initialize MC"
	Button button0,pos={120,160},size={75,28},proc=GoMC,title="Go MC"
	
	SetVariable setvar1,pos={20,25},size={150,16},title="Alpha0,ProtA (s-1)",value= G_alpha0
	SetVariable setvar8,pos={20,45},size={150,16},title="Alpha0,ProtB (s-1)"
	SetVariable setvar8,value= G_alpha0_b
	SetVariable setvar13,pos={20,65},size={150,16},title="Alpha0,ref (s-1)",value= G_alpha0_c
	
	SetVariable setvar0,pos={175,25},size={145,16},title="DeltaX,ProtA (nm)"
	SetVariable setvar0,value= G_Dx
	SetVariable setvar9,pos={175,45},size={145,16},title="DeltaX,ProtB (nm)",value= G_Dx_b
	SetVariable setvar14,pos={175,65},size={145,16},title="DeltaX,Ref (nm)",value= G_Dx_c

	SetVariable setvar11,pos={330,25},size={152,16},title="# of events, ProtA"
	SetVariable setvar11,value= G_Number_Of_Events
	SetVariable setvar12,pos={330,45},size={152,16},title="# of events, ProtB"
	SetVariable setvar12,value= G_Number_Of_Events_b
	
	SetVariable setvar2,pos={20,85},size={100,16},title="Ramp (pN/s)"
	SetVariable setvar2,value= G_Ramp
	SetVariable setvar5,pos={20,102},size={110,16},title="# of cycles"
	SetVariable setvar5,limits={0,1000,1},value= G_Number_Of_Cycles
	SetVariable setvar7,pos={20,119},size={140,16},title="# of experiments"
	SetVariable setvar7,limits={0,1000,1},value= G_Number_Of_Experiments
	SetVariable setvar6,pos={20,136},size={185,16},title="Uncertainty in Force (RSD, %)"
	SetVariable setvar6,limits={0,20,0.1},value= G_uncertainty
	
	
	CheckBox traditional,pos={210,95},size={130,16},proc=SetTraditional,title="Traditional"
	CheckBox traditional,value= 1	

	CheckBox concurrent,pos={300,95},size={130,16},proc=SetConcurrent,title="Concurrent"
	CheckBox concurrent,value= 0	

	CheckBox internalstandard,pos={210,110},size={130,16},proc=SetInternalStandard,title="Comparison using Internal Standard"
	CheckBox internalstandard,value= 0	
	
	CheckBox inputtable,pos={210,125},size={130,16},proc=SetInputTable,title="Data from input table"
	CheckBox inputtable,value= 0	
	
	SetVariable setvar3,pos={20,200},size={140,16},title="Time step (s)"
	SetVariable setvar3,limits={0,0.1,0.001},value= G_Time_Step_Interval
	SetVariable setvar4,pos={165,200},size={100,16},title="Bin size (pN)"
	SetVariable setvar4,limits={0,25,1},value= G_Binsize


	GroupBox box2,pos={275,150},size={90,70},title=""
	GroupBox box2,font="Times New Roman",fSize=12
	DrawText 205,175,"Avg. Method"

	CheckBox Schlierf,pos={285,160},size={120,16},proc=SetSchlierf,title="Schlierf"
	CheckBox Schlierf,value=1

	CheckBox Gaussian,pos={285,180},size={120,16},proc=SetGaussian,title="Gaussian"
	CheckBox Gaussian,value= 0
	
	CheckBox Average,pos={285,200},size={120,16},proc=SetAverage,title="Mean"
	CheckBox Average,value= 0	


EndMacro

Function InitMC(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	switch( ba.eventCode )
		case 2: // mouse up
			InitializeMC()
			break
	endswitch
	
	return 0
End

Function GoMC(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			MonteCarlo()
			break
	endswitch

	return 0
End

Function SetTraditional(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 
	NVAR mode = root:G_mode
	if (check==1)
	mode = 1			//variation in SC: tradidional AFM
	CheckBox Concurrent,value= 0
	else
	mode =2
	endif	

End

Function SetConcurrent(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 
	NVAR mode = root:G_mode
	NVAR internalstandard_mode = root:G_internalstandard_mode
	if (check==1)
	mode = 0			//no variation in SC within the same experiment: concurrent AFM
	CheckBox Traditional,value= 0
	CheckBox Internalstandard,value=0
	Internalstandard_mode=0
	else
	mode =2
	endif

End

Function SetInternalStandard(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 
	NVAR internalstandard_mode = root:G_internalstandard_mode
	NVAR mode = root:G_mode
	if (check == 1)
	internalstandard_mode = 1			//comparison using internal standard
	CheckBox Concurrent, value=0
	CheckBox Traditional, value=1
	mode = 1
	elseif (check == 0)
	internalstandard_mode = 0
	endif	
End

Function SetInputTable(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 
	NVAR inputtable = root:G_inputtable
	if (check == 1)
	inputtable = 1			//use table to input number of experiments and number of events
	elseif (check == 0)
	inputtable = 0
	endif	
End

Function SetSchlierf(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 0 if not
	NVAR avgmethod = root:G_averagingmethod
	if (check==1)
	avgmethod = 0
	CheckBox Gaussian,value= 0
	CheckBox Average,value= 0
	else
	avgmethod=4
	endif
End

Function SetGaussian(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 0 if not
	NVAR avgmethod = root:G_averagingmethod
	if (check==1)
	avgmethod = 1
	CheckBox Schlierf,value= 0
	CheckBox Average,value= 0
	else
	avgmethod=4
	endif
End

Function SetAverage(ctrlName,check) : CheckBoxControl
	String ctrlName
	Variable check		// 1 if selected 0 if not
	NVAR avgmethod = root:G_averagingmethod
	if (check==1)
	avgmethod = 2
	CheckBox Schlierf,value=0
	CheckBox Gaussian,value=0
	else
	avgmethod=4				
	endif
End


Function MonteCarlo()

NVAR kT=G_Thermal
NVAR F=G_Force     					
NVAR alpha0=G_alpha0
NVAR alpha0_b=G_alpha0_b
NVAR alpha0_c=G_alpha0_c
NVAR dX=G_Dx
NVAR dX_b=G_Dx_b
NVAR dX_c=G_Dx_c
NVAR Dt=G_Time_Step_Interval
NVAR a=G_Ramp
NVAR NC=G_Number_Of_Cycles
NVAR n=G_Number_Of_Events
NVAR n_b=G_Number_Of_Events_b
NVAR nexperiments=G_Number_Of_Experiments
NVAR binsize=G_Binsize

NVAR w2 = G_uncertainty

NVAR mode = root:G_mode
NVAR avgmethod = root:G_averagingmethod
NVAR inputtable = root:G_inputtable
NVAR internalstandard_mode = root:G_internalstandard_mode

Make/O/N=(NC) MaxFu 			 	//maximum of the distribution for each simulated AFM experiment (mean unfolding force), 1st protein
Make/O/N=(NC) MaxFu_b 				// idem 2nd protein
Make/O/N=(NC) Diff_MaxFu 			//difference in mean unfolding force

Make/O/N=(NC) MaxFu_ref_a 			 //maximum of the distribution for each simulated AFM experiment (mean unfolding force), reference to ProtA
Make/O/N=(NC) MaxFu_ref_b 			// idem for reference to ProtB
Make/O/N=(NC) Diff_MaxFu_corr		// after normalization by internal standard 

WAVE neventsProtA, neventsProtB

variable Pu 						//probability of unfolding
variable i=0
variable j=0
variable k = 0
variable l = 0
variable num, num_b, numexperiments		//to remember number of events and experiments for panel
variable n_ref_a, n_ref_b


variable CS,error								//variation coming from calibration, ProtA
variable CS_b,error_b							//variation coming from calibration, ProtB

variable MaxFu_ideal_a, MaxFu_ideal_b, MaxFu_ideal_c		//values obtained directly from input parameters according to selected averaging method

variable nexpProtA, nexpProtB


if (avgmethod == 4)

print "Please select Averaging Method"

elseif (mode == 2)

print "Please select Traditional or Concurrent Spectroscopy"

else
	//calculate reference mFu according to selected averaging method
	// Schlierf
	if (avgmethod == 0)
		MaxFu_ideal_a= 1/(dx/4.11)*ln(dx/4.11*a/alpha0) 
		MaxFu_ideal_b = 1/(dx_b/4.11)*ln(dx_b/4.11*a/alpha0_b) 
		MaxFu_ideal_c = 1/(dx_c/4.11)*ln(dx_c/4.11*a/alpha0_c) 
	
	//Gauss	
	elseif (avgmethod ==1 )
		Make/O/N=1000 P_Schlierf, P_Schlierf_b, P_Schlierf_c
		Setscale/I x, 0, 250, P_Schlierf, P_Schlierf_b, P_Schlierf_c
	
		P_Schlierf = (alpha0/a)*(exp(x*dx/4.11))*(exp((-alpha0*4.11/(a*dx))*(exp(x*dx/4.11)-1)))
		P_Schlierf_b = (alpha0_b/a)*(exp(x*dx_b/4.11))*(exp((-alpha0_b*4.11/(a*dx_b))*(exp(x*dx_b/4.11)-1)))
		P_Schlierf_c = (alpha0_c/a)*(exp(x*dx_c/4.11))*(exp((-alpha0_c*4.11/(a*dx_c))*(exp(x*dx_c/4.11)-1)))
		
		Make/O/D/N=4/O W_coef
		CurveFit/N/Q/NTHR=0/TBOX=0 gauss  P_Schlierf /D 
		MaxFu_ideal_a = W_Coef[2] 
	
		Make/O/D/N=4/O W_coef
		CurveFit/N/Q/NTHR=0/TBOX=0 gauss  P_Schlierf_b/D 
		MaxFu_ideal_b = W_Coef[2] 
		
		Make/O/D/N=4/O W_coef
		CurveFit/N/Q/NTHR=0/TBOX=0 gauss  P_Schlierf_c/D 
		MaxFu_ideal_c = W_Coef[2] 


	//average
	else
		Make/O/N=1000 P_Schlierf, P_Schlierf_b, P_Schlierf_c
		Setscale/I x, 0, 250, P_Schlierf
		Setscale/I x, 0, 250, P_Schlierf_b
		Setscale/I x, 0, 250, P_Schlierf_c
	
		P_Schlierf = (alpha0/a)*(exp(x*dx/4.11))*(exp((-alpha0*4.11/(a*dx))*(exp(x*dx/4.11)-1)))
		P_Schlierf_b = (alpha0_b/a)*(exp(x*dx_b/4.11))*(exp((-alpha0_b*4.11/(a*dx_b))*(exp(x*dx_b/4.11)-1)))
		P_Schlierf_c = (alpha0_c/a)*(exp(x*dx_c/4.11))*(exp((-alpha0_c*4.11/(a*dx_c))*(exp(x*dx_c/4.11)-1)))
		
		wavestats/Q P_schlierf
		MaxFu_ideal_a = V_avg
	
		wavestats/Q P_schlierf_b
		MaxFu_ideal_b = V_avg
		
		wavestats/Q P_schlierf_c
		MaxFu_ideal_c = V_avg
	
	endif
	
	if (inputtable == 0)		//create table from data in Monte Carlo Panel
	neventsProtA = nan
	neventsProtB = nan
	Redimension/N=(nexperiments) , neventsProtA
	Redimension/N=(nexperiments), neventsProtB
		for (l=0; l<nexperiments; l+=1)
		neventsProtA[l] = n
		neventsProtB[l] = n_b
		endfor
	
		
	endif					//if input table is active, we take values from table directly
		
	
	num=n					//to recover values for panel
	num_b=n_b
	numexperiments = nexperiments
			
	
	
	wavestats/Q neventsProtA						//to find maximum number of events from experiments
	nexpProtA = V_npnts
	n=V_max
	
	wavestats/Q neventsProtB
	nexpProtB = V_npnts
	n_b=V_max
	
	if (inputtable ==1  && mode == 0 && nexpProtB !=nexpProtA)
	
		print "Concurrent spectroscopy is not compatible with different number of experiments. Please correct"
	
		
	else
	
	
		if (nexpProtA > nexpProtB)									//so that neventsProtA and neventsProtB have the same dimensions
		nexperiments = nexpProtA					
		Redimension/N=(nexperiments), neventsProtB 
		elseif (nexpProtA < nexpProtB)
		nexperiments = nexpProtB
		Redimension/N=(nexperiments), neventsProtA 
		else 
		nexperiments = nexpProtA
		endif
	
						//next lines: to create Fu and Fu_b waves with the right dimensions. Extra rows do not interfere (they are left as nan)
	
		if (n>=n_b)
		Make/O/N=(n,nexpProtA) Fu 	//number of events: number or rows; number of experiments: number of columns; ProtA
		Make/O/N=(n,nexpProtB) Fu_b  //idem, ProtB
		else
		Make/O/N=(n_b,nexpProtA) Fu 	//number of events: number or rows; number of experiments: number of columns; ProtA
		Make/O/N=(n_b,nexpProtB) Fu_b  //idem, ProtB
		endif
	
	Fu =nan
	Fu_b=nan
	
		If (internalstandard_mode ==1)
	
		Duplicate/O Fu, Fu_ref_a					//create waves to store reference unfolding forces if using internal standard
		Duplicate/O Fu_b, Fu_ref_b
		
		endif	
	
	
	For (j=0; j<NC;j+=1)	// MC cycles to get distributions for a set of conditions
				
		For (i=0; i<nexperiments; i+=1) //obtain data from the distribution of experiments
	
	
			CS = 100 + gnoise(w2)							
			error = CS/100
	
			if (mode==1)										//decide wheter variation in spring constant is considered (traditional) or not (concurrent)		
			CS_b = 100 + gnoise(w2)							//if concurrent: error is the same; if traditional: second protein has different error
			error_b = CS_b/100
			else
			error_b=error
			endif
			

			n = neventsProtA[i]
			n_b = neventsProtB[i]
			
			if (n > 0)
				
				Do														//generates distribution of unfolding forces for protA 
				Pu = n*alpha0*exp (F*error*dX/kT)*Dt				//number of domains(events) * probability of unfolding of a domain
																		//real forces and ramps according to error 
					if (Pu<(enoise(1) + 1)/2)							//random number
			
					else
					
					Fu[n-1] [i]=F									//if Pu is higher than random number, then store measured force and we have one less domain to unfold
					n -=1
		
		 			endif
				
				k+=1									//to keep track of force during ramp
				F=a*k*Dt								//update force
		
				
				while (n>0)								//stop when there are no more domains to unfold
		
			F = 	0						//to start over ramp
			k = 0	
			
			endif
			
			if (n_b >0)
			
				Do															//generates distribution of unfolding forces for ProtB
				Pu = n_b*alpha0_b*exp (F*error_b*dX_b/kT)*Dt			//number of domains(events) * probability of unfolding of a domain
																			//real forces and ramps according to error 
					if (Pu<(enoise(1) + 1)/2)								//random number
			
					else
					
					Fu_b[n_b-1] [i]=F										//if Pu is higher than random number, then store measured force and we have one less domain to unfold
					n_b -=1
		
		 			endif
				
				k+=1									//to keep track of force during ramp
				F=a*k*Dt								//update force
		
				
				while (n_b>0)								//stop when there are no more domains to unfold
		
		
				

			F = 	0						//to start over ramp
			k = 0
			endif
			
			if (mode==1 && internalstandard_mode == 1)					//internal standard correction
			
			n_ref_a = neventsProtA[i]										//number of events for reference to ProtA, equal to number of events for ProtA
			n_ref_b = neventsProtB[i]										//number of events for reference to ProtB, equal to number of events for ProtB
			
					if (n_ref_a >0)
			
					Do																//generates distribution of unfolding forces for reference to ProtA - same error in calibration as ProtA
					Pu = n_ref_a*alpha0_c*exp (F*error*dX_c/kT)*Dt			//number of domains(events) * probability of unfolding of a domain
																					//real forces and ramps according to error 
						if (Pu<(enoise(1) + 1)/2)									//random number
				
						else
						
						Fu_ref_a[n_ref_a-1] [i]=F								//if Pu is higher than random number, then store measured force and we have one less domain to unfold
						n_ref_a -=1
			
			 			endif
					
					k+=1									//to keep track of force during ramp
					F=a*k*Dt								//update force
			
					
					while (n_ref_a>0)								//stop when there are no more domains to unfold
			
			
					
	
					F = 	0						//to start over ramp
					k = 0
					endif
				
			
					if (n_ref_b >0)
			
					Do																//generates distribution of unfolding forces for reference to ProtB - same error in calibration as ProtB
					Pu = n_ref_b*alpha0_c*exp (F*error_b*dX_c/kT)*Dt			//number of domains(events) * probability of unfolding of a domain
																					//real forces and ramps according to error 
						if (Pu<(enoise(1) + 1)/2)									//random number
				
						else
						
						Fu_ref_b[n_ref_b-1] [i]=F							//if Pu is higher than random number, then store measured force and we have one less domain to unfold
						n_ref_b -=1
			
			 			endif
					
					k+=1									//to keep track of force during ramp
					F=a*k*Dt								//update force
			
					
					while (n_ref_b>0)								//stop when there are no more domains to unfold
			
			
					
	
					F = 	0						//to start over ramp
					k = 0
					endif
			
			
			
			
			endif
		
		
		
		
		
		endfor
	

	
		Make/N=10/O Fu_Hist;DelayUpdate 	//calculate histogram with all unfolding events from nexperiments, ProtA
		Histogram/P/B={0,binsize,250/binsize} Fu,Fu_Hist
	
		Make/N=10/O Fu_Hist_b;DelayUpdate 	//calculate histogram with all unfolding events from nexperiments, ProtB 
		Histogram/P/B={0,binsize,250/binsize} Fu_b,Fu_Hist_b
	
		if (internalstandard_mode == 1)
		Make/N=10/O Fu_ref_a_Hist;DelayUpdate 	//calculate histogram with all unfolding events from nexperiments, ref for ProtA
		Histogram/P/B={0,binsize,250/binsize} Fu_ref_a,Fu_ref_a_Hist
	
		Make/N=10/O Fu_ref_b_Hist;DelayUpdate 	//calculate histogram with all unfolding events from nexperiments, ref for ProtB
		Histogram/P/B={0,binsize,250/binsize} Fu_ref_b,Fu_ref_b_Hist
		endif
		
		// Use fitting parameters to obtain MaxFu (or mean unfolding force) by Schlierf fitting or Gauss Fitting, or directly from averaging
	
		// Schlierf
		if (avgmethod == 0)
		Make/O/D/N=3/O W_coef
		W_coef[0] = {0.01,0.2,40}
		
		FuncFit/H="001"/Q/N=1/NTHR=0/TBOX=0 Schlierf W_coef  Fu_Hist /D 
		MaxFu [j] = 1/(W_coef[1]/4.11)*ln(W_coef[1]/4.11*a/W_coef[0]) 
	
		FuncFit/H="001"/Q/N=1/NTHR=0/TBOX=0 Schlierf W_coef  Fu_Hist_b /D 
		MaxFu_b [j] = 1/(W_coef[1]/4.11)*ln(W_coef[1]/4.11*a/W_coef[0]) 
		
		if (internalstandard_mode == 1)
		FuncFit/H="001"/Q/N=1/NTHR=0/TBOX=0 Schlierf W_coef  Fu_ref_a_Hist/D 
		MaxFu_ref_a [j] = 1/(W_coef[1]/4.11)*ln(W_coef[1]/4.11*a/W_coef[0]) 
		
		FuncFit/H="001"/Q/N=1/NTHR=0/TBOX=0 Schlierf W_coef  Fu_ref_b_Hist/D 
		MaxFu_ref_b [j] = 1/(W_coef[1]/4.11)*ln(W_coef[1]/4.11*a/W_coef[0]) 
		endif
	
		//Gauss	
		elseif (avgmethod ==1 )
		Make/O/D/N=4/O W_coef
		CurveFit/N/Q/NTHR=0/TBOX=0 gauss  Fu_Hist /D 
		MaxFu [j] = W_Coef[2]
		
		CurveFit/N/Q/NTHR=0/TBOX=0 gauss  Fu_Hist_b /D 
		MaxFu_b [j] = W_Coef[2]
			
			if (internalstandard_mode == 1)
			CurveFit/N/Q/NTHR=0/TBOX=0 gauss  Fu_ref_a_Hist /D 
			MaxFu_ref_a [j] = W_Coef[2]
			
			CurveFit/N/Q/NTHR=0/TBOX=0 gauss  Fu_ref_b_Hist /D 
			MaxFu_ref_b [j] = W_Coef[2]
			endif		
				
		//average
		else
		wavestats/Q Fu
		MaxFu [j] = V_avg
		
		wavestats/Q Fu_b
		MaxFu_b [j] = V_avg
		
			if (internalstandard_mode == 1)
			wavestats/Q Fu_ref_a_Hist
			MaxFu_ref_a [j] = V_avg
			
			wavestats/Q Fu_ref_b_Hist
			MaxFu_ref_b [j] = V_avg
			endif
					
		endif
			


	endfor
	
		

	
	
	n=num						//recover number of events for panel
	n_b=num_b
	nexperiments = numexperiments
	
	if (internalstandard_mode == 0)
	
	Diff_MaxFu = MaxFu - MaxFu_b
	
	else 
	
	Diff_MaxFu = MaxFu - MaxFu_b
	Diff_MaxFu_corr = MaxFu * (MaxFu_ref_a /MaxFu_ideal_c)^-1 - MaxFu_b * (MaxFu_ref_b /MaxFu_ideal_c)^-1
	

	endif
		
	Make/N=100/O MaxFu_Hist;DelayUpdate						//distribution of mean unfolding forces, ProtA
	Histogram/B={50,0.5,200} MaxFu,MaxFu_Hist;DelayUpdate
	
	Make/N=100/O MaxFu_Hist_b;DelayUpdate						//distribution of mean unfolding forces, ProtB
	Histogram/B={50,0.5,200} MaxFu_b,MaxFu_Hist_b;DelayUpdate
	
	Make/N=100/O Diff_MaxFu_Hist;DelayUpdate						//distribution of mean unfolding forces, difference
	Histogram/B={-50,0.5,200} Diff_MaxFu,Diff_MaxFu_Hist;DelayUpdate

		if (internalstandard_mode == 1)
		Make/N=100/O Diff_MaxFu_corr_Hist;DelayUpdate						//distribution of mean unfolding forces, difference corrected by internal standard
		Histogram/B={-50,0.5,200} Diff_MaxFu_corr,Diff_MaxFu_corr_Hist;DelayUpdate	
		endif
	
	wavestats/Q MaxFu
	variable average, SD
	average = V_avg
	SD =  V_sdev
	
	wavestats/Q MaxFu_b
	variable average_b, SD_b
	average_b = V_avg
	SD_b =  V_sdev
	
	wavestats/Q Diff_MaxFu
	variable average_diff, SD_diff
	average_diff = V_avg
	SD_diff =  V_sdev
	
		if (internalstandard_mode == 1)
		
		wavestats/Q MaxFu_ref_a
		variable average_ref_a, SD_ref_a
		average_ref_a= V_avg
		SD_ref_a =  V_sdev
		
		wavestats/Q MaxFu_ref_b
		variable average_ref_b, SD_ref_b
		average_ref_b = V_avg
		SD_ref_b =  V_sdev
		
		wavestats/Q Diff_MaxFu_corr
		variable average_diff_corr, SD_diff_corr
		average_diff_corr = V_avg
		SD_diff_corr =  V_sdev
		
		endif
	
	print "****************************************************"
	
	print "Simulation done on", DateTimeStamp()
	print "Type of Atomic Force Spectroscopy simulated:"
	
		if (mode == 0)
		print "1. Concurrent. Uncertainty in Force (RSD, in %)", w2
		else
		print "1. Traditional. Uncertainty in Force (RSD, in %)", w2
		endif
		
		if (internalstandard_mode == 0)
		print "2. No Correction by Internal Standard"
		else
		print "2. Corrected by Internal Standard"
		endif
	
		if (avgmethod == 0)
		print "3. Schlierf averaging"
		elseif (avgmethod == 1)
		print "3. Gaussian averaging"
		elseif (avgmethod == 2)
		print "3. Mean averaging"
		endif
	
	
	print "***********"
	print "Summary of settings:"
	print "ramp: ", a, " (pN/s)" 
	print "Alpha0_a:", alpha0, "deltaX_a: ", dX, ". Theoretical mean Fu, ProtA: ", MaxFu_ideal_a, "(pN)." 
	print "Alpha0_b:", alpha0_b, "deltaX_b: ", dX_b, ". Theoretical mean Fu, ProtB: ", MaxFu_ideal_b, "(pN)."
	
	if (internalstandard_mode == 1)
	print "Internal standard parameters: Alpha0_c:", alpha0_c, "deltaX_c: ", dX_c, "Theoretical mean Fu: ", MaxFu_ideal_c, "(pN)." 
	endif
	
	print "Number of cycles", NC
	print "Time step MC: ", Dt
	print "Binsize: ", binsize,  " pN" 

	if (inputtable==1)
	print "Number of events fron input table"
	print "Number of experiments (ProtA):", nexpProtA, neventsProtA
	print "Number of experiments (ProtB):", nexpProtB, neventsProtB
	
	elseif (inputtable==0)
		if (mode == 0)
		print "Number of events from MC panel"
		print "Number of events (ProtA)", n 
		print "Number of events (ProtB)", n_b 
		print "Number of experiments:", nexperiments
		else
		print "Number of events from MC panel"
		print "Number of events (ProtA)", n 
		print "Number of events (ProtB)", n_b 
		print "Number of experiments in panel:", nexperiments
		print "Number of experiments in real life: ", 2*nexperiments
		endif
	endif
	
	print "***********"	
	print "RESULTS:"
	print "Mean distribution of mean Fu, ProtA: ", average, "(pN).      SD mean Fu_ProtA: ", SD, "(pN).		RSD: ", SD/average
	print "Mean distribution of mean Fu, ProtB: ", average_b, "(pN).      SD mean Fu_ProtB: ", SD_b, "(pN).		RSD: ", SD_b/average_b
	print "Mean distribution of Delta mean Fu: ", average_diff, "(pN).      SD mean mean Fu: ", SD_diff, "(pN)		RSD: ", SD_diff/MaxFu_ideal_a 
	if (internalstandard_mode == 1)
	print "Mean distribution of mean Fu, reference to ProtA: ", average_ref_a, "(pN).      SD mean Fu, reference to ProtA: ", SD_ref_a, "(pN).			RSD: ", SD_ref_a/average_ref_a
	print "Mean distribution of mean Fu, reference to ProtB: ", average_ref_b, "(pN).      SD mean Fu, reference to ProtB: ", SD_ref_b, "(pN).		RSD: ", SD_ref_b/average_ref_b
	print "Corrected mean distribution Delta mean Fu by internal standard: ", average_diff_corr, "(pN).     Corrected SD Delta Mean Fu: ", SD_diff_corr, "(pN)		RSD: ", SD_diff_corr/MaxFu_ideal_a 
	endif
	print "****************************************************"	
	
	DoWindow/F Distribution_mFu
		if(V_Flag==1)	// if there is a window, kill it
			DoWindow/K Distribution_mFu
		endif
	Display/W=(0,0,550,300)
	DoWindow/C Distribution_mFu
	SetWindow Distribution_mFu,hook(testhook)=KeyboardPanelHook 
	appendtograph MaxFu_Hist, MaxFu_Hist_b, Diff_MaxFu_Hist
	if (internalstandard_mode == 1)
	appendtograph Diff_MaxFu_corr_hist
	endif
	ModifyGraph mode=6,lsize=1.5,rgb=(0,0,0)
	Label left "# of events";DelayUpdate
	Label bottom "mFu or Delta (mFu)"
	
	DoWindow/F Values_mFu
		if(V_Flag==1)	// if there is a window, kill it
			DoWindow/K Values_mFu
		endif
	Display/W=(000,400,550,700)
	DoWindow/C Values_mFu
	SetWindow Values_mFu,hook(testhook)=KeyboardPanelHook 
	appendtograph MaxFu, MaxFu_b, Diff_MaxFu
	if (internalstandard_mode == 1)
	appendtograph Diff_MaxFu_corr
	ModifyGraph rgb(Diff_MaxFu_corr)=(16386,65535,16385)
	appendtograph MaxFu_ref_a, MaxFu_ref_b
	ModifyGraph rgb(MaxFu_ref_a)=(0,65535,0),rgb(MaxFu_ref_b)=(0,65535,0)
	endif
	ModifyGraph mode=0,lsize=1.5; DelayUpdate
	ModifyGraph rgb(MaxFu_b)=(65280,0,0); DelayUpdate
	Label left "mFu or Delta (mFu)";DelayUpdate
	Label bottom "Cycle #"
		
	endif	

endif

beep

End

//********************************************************************************************************************************
commentize if already in igor
Function Schlierf(w,F) : FitFunc
	Wave w
	Variable F

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(F) = (alpha0/a)*(exp(F*dx/4.11))*(exp((-alpha0*4.11/(a*dx))*(exp(F*dx/4.11)-1)))
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ F
	//CurveFitDialog/ Coefficients 3
	//CurveFitDialog/ w[0] = alpha0
	//CurveFitDialog/ w[1] = dX
	//CurveFitDialog/ w[2] = a

	return (w[0]/w[2])*(exp(F*w[1]/4.11))*(exp((-w[0]*4.11/(w[2]*w[1]))*(exp(F*w[1]/4.11)-1)))
End

//********************************************************************************************************************************

//Obtained from https://www.wavemetrics.com/code-snippet/return-datetime-stamp

Function/S DateTimeStamp()
    String dstr, tstr
   
    Variable timestamp = DateTime
    String sep_char = "."
   
    tstr = Secs2Time(timestamp, 3)
    dstr = Secs2Date(timestamp, -2, sep_char)
   
    // Replace numeric month with text abbreviation.
    String months = "Jan;Feb;Mar;Apr;May;Jun;Jul;Aug;Sep;Oct;Nov;Dec;"
    String month_abbreviation = StringFromList(str2num(StringFromList(1, dstr, sep_char)) - 1, months, ";")
    dstr = RemoveListItem(1, dstr, sep_char)
    dstr = AddListItem(month_abbreviation, dstr, sep_char, 1)
    dstr = RemoveEnding(dstr, sep_char)
    return (dstr + "|" + tstr)
End
